/*
 * globals.c
 *
 *  Created on: Apr 18, 2021
 *      Author: 15077
 */

#include "globals.h"
#include "log.h"
#include "util.h"

int g_usb30_INT_count[MAX_USB30_DBG_COUNT];
int g_usb30_INT_count_last[MAX_USB30_DBG_COUNT];
uint32_t g_tickCount = 0;


void InitUSB30Debug()
{

    for(int i=0; i < MAX_USB30_DBG_COUNT; i++ )
    {
        g_usb30_INT_count_last[i] = g_usb30_INT_count[i];
    }
}

void PrintUSB30IntStats()
{

    int temp[MAX_USB30_DBG_COUNT];
    for(int i = 0; i < MAX_USB30_DBG_COUNT; i++)
    {
        temp[i] = g_usb30_INT_count[i] - g_usb30_INT_count_last[i];
        g_usb30_INT_count_last[i] = g_usb30_INT_count[i];
    }

    for(int i = 0; i < 8; i++)
    {
        uart_printf("USB30 PIPE%d IN %04d, OUT %04d\r\n",
        i,
        temp[USB30_PIPE0_IN  + i],
        temp[USB30_PIPE0_OUT + i]);
    }

    uart_printf("ITP INT=%d\r\n",temp[USB30_ITP_INT]);

    uart_printf("USB30_DBG_COUNT1=%d\r\n",temp[USB30_DBG_COUNT1]);
    uart_printf("USB30_DBG_COUNT2=%d\r\n",temp[USB30_DBG_COUNT2]);
    uart_printf("USB30_DBG_COUNT3=%d\r\n",temp[USB30_DBG_COUNT3]);
    uart_printf("USB30_DBG_COUNT4=%d\r\n",temp[USB30_DBG_COUNT4]);
    uart_printf("USB30_DBG_COUNT5=%d\r\n",temp[USB30_DBG_COUNT5]);
    uart_printf("USB30_DBG_COUNT6=%d\r\n",temp[USB30_DBG_COUNT6]);
    uart_printf("USB30_DBG_COUNT7=%d\r\n",temp[USB30_DBG_COUNT7]);
    uart_printf("USB30_DBG_COUNT8=%d\r\n",temp[USB30_DBG_COUNT8]);
}


typedef struct _SSLinkLogEntry
{
	UINT64 sysTick;
	UINT32 no;
	UINT32 Intflag;
	UINT32 errStatus;
	UINT32 linkIntReg;

}SSLinkLogEntry;


#define MAX_LINK_LOG_CAPACITY 1000

SSLinkLogEntry g_SS_Link_Log_Buf[MAX_LINK_LOG_CAPACITY] __attribute__((section(".DMADATA")));

uint32_t g_link_log_WritePos = 0;
uint32_t g_link_log_ReadPos  = 0;
uint32_t g_link_log_print_no =0;

void USB30LinkLog(uint32_t linkIntFlag, uint32_t errStatus, uint32_t linkIntReg)
{

	//if(g_link_log_WritePos < MAX_LINK_LOG_CAPACITY)
	{
		uint32_t index = g_link_log_WritePos;

		uint32_t nextWritePos = (g_link_log_WritePos + 1);

		if(nextWritePos == MAX_LINK_LOG_CAPACITY)
		{
			nextWritePos = 0;
		}

		if(nextWritePos != g_link_log_ReadPos)
		{//ȷдδ
			SSLinkLogEntry* pEntry = (SSLinkLogEntry*)&g_SS_Link_Log_Buf[index];
			pEntry->Intflag   = linkIntFlag;
			pEntry->sysTick    = GetSysTick();
			pEntry->no          = g_link_log_print_no;
			pEntry->errStatus  = errStatus;
			pEntry->linkIntReg = linkIntReg;


			g_link_log_WritePos++;
			if(g_link_log_WritePos == MAX_LINK_LOG_CAPACITY)
			{
				g_link_log_WritePos = 0;
			}
		}
	}

	g_link_log_print_no ++;
}


#include "CH56x_usb30_lib.h"
const char* GetLinkIntString(UINT32 IntFlag)
{
    const char* str = "";
    switch(IntFlag)
    {

    case LINK_RDY_FLAG://           (1<<0)

        str = "LINK_RDY_FLAG";
        break;

    case  LINK_RECOV_FLAG://         (1<<1)
        str = "LINK_RECOV_FLAG";
        break;

    case  LINK_INACT_FLAG://         (1<<2)
        str = "LINK_INACT_FLAG";
        break;
    case  LINK_DISABLE_FLAG://       (1<<3)
        str = "LINK_DISABLE_FLAG";
        break;
    case  LINK_GO_U3_FLAG://         (1<<4)
        str = "LINK_GO_U3_FLAG";
        break;
    case  LINK_GO_U2_FLAG://         (1<<5)
        str = "LINK_GO_U2_FLAG";
        break;
    case  LINK_GO_U1_FLAG://         (1<<6)
        str = "LINK_GO_U1_FLAG";
        break;
    case  LINK_GO_U0_FLAG://         (1<<7)
        str = "LINK_GO_U0_FLAG";
        break;
    case  LINK_U3_WAKE_FLAG://       (1<<8)
        str = "LINK_U3_WAKE_FLAG";
        break;
    case  LINK_Ux_REJECT_FLAG://     (1<<9)
        str = "LINK_Ux_REJECT_FLAG";
        break;
    case  TERM_PRESENT_FLAG://       (1<<10)
        str = "TERM_PRESENT_FLAG";
        break;
    case  LINK_TXEQ_FLAG://          (1<<11)
        str = "LINK_TXEQ_FLAG";
        break;
    case  LINK_Ux_EXIT_FLAG://       (1<<12)
        str = "LINK_Ux_EXIT_FLAG";
        break;
    case  WARM_RESET_FLAG://         (1<<13)
        str = "WARM_RESET_FLAG";
        break;
    case  U3_WAKEUP_FLAG://          (1<<14)
        str = "U3_WAKEUP_FLAG";
        break;
    case  HOT_RESET_FLAG://          (1<<15)
        str = "HOT_RESET_FLAG";
        break;
    case  LINK_RX_DET_FLAG://       (1<<20)
        str = "LINK_RX_DET_FLAG";
        break;
    }//switch

    return str;
}


void PrintUSB30LinkLog()
{
	while(g_link_log_ReadPos != g_link_log_WritePos)
	{//λòдλãζ־δӡ
		uint32_t index = g_link_log_ReadPos;
		SSLinkLogEntry* pEntry = (SSLinkLogEntry*)&g_SS_Link_Log_Buf[index];

		UINT64 sysTick = 0xFFFFFFFFFFFFFFFF - pEntry->sysTick;

		UINT32 deltaT = sysTick*1000000/SYS_CLK;

		uart_printf("[LINK %04d](%8d.%03dms),ERR_STATUS=0x%08x, LINK_INT_REG=0x%08x, %s\r\n", pEntry->no,  deltaT/1000, deltaT%1000, pEntry->errStatus, pEntry->linkIntReg, GetLinkIntString(pEntry->Intflag));


		g_link_log_ReadPos ++;
		if(g_link_log_ReadPos == MAX_LINK_LOG_CAPACITY)
		{
			g_link_log_ReadPos = 0;
		}
	}

}
