#include "CH56x_common.h"
#include <stdarg.h>
#include "log.h"
#include "globals.h"
//char g_LogBuf[8192+4096];
//char g_LogBuf[8192];
//uint8_t g_LogBuf[4*1024] __attribute__((section(".DMADATA"))); //log buffer
uint8_t g_LogBuf[40*1024] __attribute__((section(".DMADATA"))); //log buffer
uint16_t g_LogReadPos  = 0;
uint16_t g_LogWritePos = 0;
uint16_t g_OverWriteCount = 0;

enum LOG_LEVEL g_LogLevel = LOG_LEVEL_VERBOSE;
//enum LOG_LEVEL g_LogLevel = LOG_LEVEL_WARN;

#ifndef DBG_UART
#define DBG_UART Debug_UART1
#endif
int uart_printf (const char *fmt, ...)
{
  char buffer[256], *sptr = buffer;
  va_list va;
  int rslt;
  va_start(va, fmt);
  rslt = vsnprintf(buffer, sizeof(buffer), fmt, va);
  va_end(va);

  while(*sptr)
  {

      switch(DBG_UART)
      {

          case Debug_UART0:
          if(R8_UART0_TFC != UART_FIFO_SIZE)
          {
              R8_UART0_THR = *sptr++;
          }
          break;


          case Debug_UART1:
          if(R8_UART1_TFC != UART_FIFO_SIZE)
          {
              R8_UART1_THR = *sptr++;
          }
          break;


          case Debug_UART2:
          if(R8_UART2_TFC != UART_FIFO_SIZE)
          {
              R8_UART2_THR = *sptr++;
          }

          break;

          case Debug_UART3:
          if(R8_UART3_TFC != UART_FIFO_SIZE)
          {
              R8_UART3_THR = *sptr++;
          }

          break;
      }//switch
   }//while
  return rslt;
}

UINT16 GetOverwriteCount()
{
    return g_OverWriteCount;
}
void InitLog()
{
    memset(g_LogBuf, 0 , sizeof(g_LogBuf));
    g_LogReadPos = 0;
    g_LogWritePos = 0;
}

void WriteLogx(const char *fmt, ...)
{
    return;
}
void WriteLog(const char *fmt, ...)
{
     char buffer[256], *sptr = buffer;

     va_list va;
     int rslt;

    if(g_LogWritePos == sizeof(g_LogBuf)) return;


     sptr = buffer;
     va_start(va, fmt);
     rslt = vsnprintf(buffer, sizeof(buffer), fmt, va);
     va_end(va);

     while(*sptr)
     {
         g_LogBuf[g_LogWritePos] = *sptr;
         sptr ++;
         g_LogWritePos ++;

         if(g_LogWritePos == sizeof(g_LogBuf))
         {
             g_LogWritePos = 0;
             g_OverWriteCount++;
         }

         /*
         if(g_LogWritePos == sizeof(g_LogBuf))
         {
             return;
         }*/
     }//while

}

void LevelLog(enum LOG_LEVEL logLevel, const char *fmt, ...)
{
    if(logLevel < g_LogLevel) return;


    char buffer[256], *sptr = buffer;

     va_list va;
     int rslt;

    if(g_LogWritePos == sizeof(g_LogBuf)) return;


     sptr = buffer;


     switch(logLevel)
     {
     case LOG_LEVEL_VERBOSE:
         //sprintf(sptr, "[INF](%08d)",g_ClkCount);
    	 strcpy(sptr, "[INF]");
         break;

     case LOG_LEVEL_WARN:
    	 //sprintf(sptr, "[WRN](%08d)",g_ClkCount);
    	 strcpy(sptr, "[WRN]");
         break;

     case LOG_LEVEL_ERROR:
    	 //sprintf(sptr, "[ERR](%08d)",g_ClkCount);
    	 strcpy(sptr, "[ERR]");
         break;
     }

     int offset = strlen(sptr);

     va_start(va, fmt);
     rslt = vsnprintf(buffer + offset, sizeof(buffer) - offset, fmt, va);
     va_end(va);

     while(*sptr)
     {
         g_LogBuf[g_LogWritePos] = *sptr;
         sptr ++;
         g_LogWritePos ++;

         if(g_LogWritePos == sizeof(g_LogBuf))
         {
             g_LogWritePos = 0;
             g_OverWriteCount++;
         }

         /*
         if(g_LogWritePos == sizeof(g_LogBuf))
         {
             return;
         }*/
     }//while

}


void PrintLog()
{

    uint8_t bLoop = TRUE;
    while(bLoop && g_LogReadPos != g_LogWritePos && g_LogReadPos < sizeof(g_LogBuf))
    {
        switch(DBG_UART)
        {

         case Debug_UART0:
            if(R8_UART0_TFC != UART_FIFO_SIZE)
            {
                R8_UART0_THR =  g_LogBuf[g_LogReadPos];

                g_LogReadPos ++;

                if(g_LogReadPos == sizeof(g_LogBuf))
                {
                    g_LogReadPos = 0;
                    bLoop = FALSE;
                }
            }
                break;

        case Debug_UART1:
            if(R8_UART1_TFC != UART_FIFO_SIZE)
            {
                R8_UART1_THR =  g_LogBuf[g_LogReadPos];

                g_LogReadPos ++;

                if(g_LogReadPos == sizeof(g_LogBuf))
                {
                    g_LogReadPos = 0;
                    bLoop = FALSE;
                }
            }
            break;

        case Debug_UART2:
            if(R8_UART2_TFC != UART_FIFO_SIZE)
            {
                R8_UART2_THR =  g_LogBuf[g_LogReadPos];

                g_LogReadPos ++;

                if(g_LogReadPos == sizeof(g_LogBuf))
                {
                    g_LogReadPos = 0;
                    bLoop = FALSE;
                }
            }
            break;

        case Debug_UART3:
            if(R8_UART3_TFC != UART_FIFO_SIZE)
            {
                R8_UART3_THR =  g_LogBuf[g_LogReadPos];

                g_LogReadPos ++;

                if(g_LogReadPos == sizeof(g_LogBuf))
                {
                    g_LogReadPos = 0;
                    bLoop = FALSE;
                }
            }
            break;
        }//switch

    }//while

}


#define MAX_REG_LOG_COUNT 350
static struct TRegLogEntry g_regLog[MAX_REG_LOG_COUNT];
static int reg_log_write_pos = 0;
static int reg_log_read_pos  = 0;
static int reg_log_undread   = 0;

void LogRegister(uint32_t tag, uint32_t reg0, uint32_t reg1, uint32_t reg2, uint32_t reg3)
{
	if(reg_log_undread == MAX_REG_LOG_COUNT)
	{//δĿ
		return;
	}
	struct TRegLogEntry* pLogEntry = &g_regLog[reg_log_write_pos];
	pLogEntry->tag  = tag;
	pLogEntry->reg0 = reg0;
	pLogEntry->reg1 = reg1;
	pLogEntry->reg2 = reg2;
	pLogEntry->reg3 = reg3;

	reg_log_undread ++;

	reg_log_write_pos ++;
	if(reg_log_write_pos == MAX_REG_LOG_COUNT)
	{
		reg_log_write_pos = 0;
	}

}

void DumpRegisterLog(CustomizePrint fp)
{
   while(reg_log_undread)
   {
	   struct TRegLogEntry* pLogEntry = &g_regLog[reg_log_read_pos];

	   if(fp)
	   {
		   fp(pLogEntry->tag, pLogEntry->reg0, pLogEntry->reg1, pLogEntry->reg2, pLogEntry->reg3);
	   }
	   else
	   {

		   uart_printf("tag=0x%08x, reg0=0x%08x, reg1=0x%08x, reg2=0x%08x, reg3=0x%08x\r\n",
				   pLogEntry->tag,
				   pLogEntry->reg0,
				   pLogEntry->reg1,
				   pLogEntry->reg2,
				   pLogEntry->reg3);

	   }



	   reg_log_undread --;

		reg_log_read_pos ++;
		if(reg_log_read_pos == MAX_REG_LOG_COUNT)
		{
			reg_log_read_pos = 0;
		}
   }

}
